.selectize-input.dropdown-active { @include border-radius(3px); }
.selectize-input.dropdown-active::before {
  content: ' ';
  display: block;
  position: absolute;
  background: #f2f2f2;
  height: 1px;
  bottom: 0;
  left: 0;
  right: 0;
}
.selectize-control.multi .selectize-input > div {
  background: #eee;
  margin-right: 5px;
  padding: 0 2px;
  cursor: pointer;
  @include border-radius(3px);
}
.selectize-dropdown {
  background: #fafafa;
  border-top: 0 none;
  @include box-shadow(0px 2px 2px 2px rgba(0, 0, 0, 0.1));
}
.selectize-dropdown-content {
  max-height: 200px;
  overflow-y: auto !important;
  overflow-x: hidden !important;
}
.selectize-dropdown [data-selectable],
.selectize-dropdown .optgroup-header {
  padding: $selectizeOptionPadding;
}
.selectize-dropdown .optgroup:first-child .optgroup-header {
  border-top: 0 none;
}
.selectize-dropdown .optgroup-header {
  background: #fafafa;
  border-bottom: 1px solid #e8e8e8;
  border-top: 1px solid #e8e8e8;
  font-weight: bold;
  font-size: 0.8em;
  cursor: default;
}
.selectize-dropdown .create {
  color: #444;
}
.selectize-dropdown .active {
  background: darken(#fff, 13%);
}
.selectize-dropdown [data-selectable] .highlight {
  background: rgba(204, 100, 40, 0.4);
  @include border-radius(1px);
}
.selectize-input.disabled, .selectize-input.disabled * {
  cursor: default !important;
}

/**********************************************************
* BASIC AESTHETIC STYLES (single)                        *
**********************************************************/
.selectize-control.single .selectize-input {
  cursor: pointer;
}
.selectize-control.single .selectize-input::after {
  content: ' ';
  display: block;
  position: absolute;
  top: 50%;
  right: 15px;
  margin-top: -2px;
  width: 0;
  height: 0;
  border-style: solid;
  border-width: 5px 5px 0 5px;
  border-color: #808080 transparent transparent transparent;
}

.selectize-control.single .selectize-input.dropdown-active::after {
  margin-top: -3px;
  border-width: 0 5px 5px 5px;
  border-color: transparent transparent #808080 transparent;
}

.selectize-control.single .selectize-input.disabled {
  opacity: 0.5;
}

/**********************************************************
* BASIC AESTHETIC STYLES (multi)                         *
**********************************************************/
.selectize-control.multi .selectize-input.disabled {
  background-color: #fafafa;
}

/**********************************************************
* LAYOUT STYLES (mandatory)                              *
**********************************************************/
.selectize-control { position: relative; }
.selectize-input { overflow: hidden; position: relative; z-index: 1; }
.selectize-input:after { content: ' '; display: block; clear: left; }
.selectize-input .items { display: inline; }
.selectize-input > * {
  vertical-align: baseline;
  display: -moz-inline-stack;
  display: inline-block;
  zoom: 1;
  *display: inline;
}
.selectize-input > input {
  max-width: 100% !important;
  text-indent: 0 !important;
  border: 0 none !important;
  background: none !important;
  padding: 0 !important;
  margin: 0;
  line-height: inherit !important;
  -webkit-user-select: auto !important;
  @include box-shadow(none !important);
}
.selectize-input > input:focus { outline: none !important; }
.selectize-dropdown { position: absolute; z-index: 2; }
.selectize-dropdown [data-selectable] { cursor: pointer; overflow: hidden; }

.selectize-control.plugin-drag_drop.multi > .selectize-input > div.ui-sortable-placeholder {
  visibility: visible !important;
  background: #f2f2f2 !important;
  background: rgba(0,0,0,0.06) !important;
  border: 0 none !important;
  @include box-shadow(inset 0 0 12px 4px #fff);
}
.selectize-control.plugin-drag_drop .ui-sortable-placeholder::after {
  content: '!';
  visibility: hidden;
}
.selectize-control.plugin-drag_drop .ui-sortable-helper {
  @include box-shadow(0 2px 5px rgba(0,0,0,0.2) !important);
}
.selectize-dropdown-header { padding: 6px 9px; @include border-radius(3px 3px 0 0); }
.selectize-dropdown-header-close { float: right; color: #ddd; font-size: 20px !important; }
.selectize-dropdown-header-close:hover { color: #aaa; }
.selectize-control.plugin-optgroup_columns .optgroup { border-right: 1px solid #f2f2f2; float: left; }
.selectize-control.plugin-optgroup_columns .optgroup:last-child { border-right: 0 none; }
.selectize-control.plugin-optgroup_columns .optgroup-header { border-top: 0 none; }
.selectize-control.plugin-remove_button .item { position: relative; padding-right: 24px !important; }
.selectize-control.plugin-remove_button .item .remove {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  width: 17px;
  text-align: center;
  font-size: 12px;
  font-weight: bold;
  color: inherit;
  vertical-align: middle;
  display: inline-block;
  padding: 7px 0 0 0;
  line-height: 8px;
  border-left: 1px solid #74b21e;
  @include border-radius(0 2px 2px 0);
}
.selectize-control.plugin-remove_button .item .remove:hover {
  border-left-color: #5e8f1a;
  background: rgba(50,90,0,0.15);
}
