html { height: 100%; }

div.notification.question {
  width: 100%;
  z-index: $navBarZindex - 1;
  position: fixed;
  top: 40px;
  display: none;
  button, .button { font-size: 0.9em; }
}

hr {
  border: none;
  border-top: 1px solid #ddd;
  line-height: 1px;
  height: 1px;
  margin-top: 1em;
  padding-bottom: 1em;
}

@media (min-width: $responsiveWidth) {
  .hide-on-widescreen { display: none !important; }
}

@media (max-width: $responsiveWidth) {
  .row { padding: 0; }

  footer {
    height: $navHeight;
    input, input:focus {
      border-bottom: 0;
      border-right: 0;
      border-left: 0;
    }
  }

  dl {
    dt {
      float: none;
      overflow: hidden;
      text-overflow: ellipsis;
      white-space: nowrap;
    }
    dd { margin-left: $bodyGutter; }
  }
}
