.gist {
  color: #000;
  font-size: 0.9em;
  pre { white-space: pre; }
  .gist-file {
    border: 1px solid #ddd;
    border-bottom: 1px solid #ccc;
    border-radius: 3px;
    margin-bottom: 1em;

    .gist-meta {
      overflow: hidden;
      color: #666;
      padding: 5px 10px;
      background-color: #e8e8e8;
      background-image: -webkit-gradient(linear, left top, left bottom, from(#fafafa), to(#e8e8e8));
      background-image: -webkit-linear-gradient(#fafafa, #e8e8e8);
      background-image: linear-gradient(#fafafa, #e8e8e8);
      background-repeat: repeat-x;
      a { text-decoration: none; }
    }

    .gist-data {
      overflow: auto;
      word-wrap: normal;
      background-color: #f8f8ff;
      border-bottom: 1px solid #ddd;
      max-height: 400px;

      .line-data { padding: .5em !important; }
      .line-pre {
        background: transparent !important;
        border: none !important;
        margin: 0 !important;
        padding: 0 !important;
      }
      .gist-highlight { background: transparent !important; }
      .line-numbers {
        background-color: #ececec;
        color: #aaa;
        border-right: 1px solid #ddd;
        text-align: right;
        padding: .5em;
        .line-number { clear: right; display: block;
        }
      }
    }
  }
}

.gist-syntax {
  background: #ffffff;
  .c { color: #999988; font-style: italic; }
  .err { color: #a61717; background-color: #e3d2d2; }
  .k, .o { color: #000000; font-weight: bold; }
  .cm { color: #999988; font-style: italic; }
  .cp { color: #999999; font-weight: bold; }
  .c1 { color: #999988; font-style: italic; }
  .cs { color: #999999; font-weight: bold; font-style: italic; }
  .gd { color: #000000; background-color: #fdd; }
  .gd .x { color: #000000; background-color: #faa; }
  .ge { color: #000000; font-style: italic; }
  .gr { color: #a00; }
  .gh { color: #999; }
  .gi { color: #000000; background-color: #dfd; }
  .gi .x { color: #000000; background-color: #afa; }
  .go { color: #888; }
  .gp { color: #555; }
  .gs { font-weight: bold; }
  .gu { color: #aaa; }
  .gt { color: #a00; }
  .kc, .kd, .kp, .kr { color: #000000; font-weight: bold; }
  .kt { color: #445588; font-weight: bold; }
  .m { color: #099; }
  .s { color: #d14; }
  .na { color: teal; }
  .nb { color: #0086b3; }
  .nc { color: #445588; font-weight: bold; }
  .no { color: teal; }
  .ni { color: purple; }
  .ne, .nf { color: #990000; font-weight: bold; }
  .nn { color: #555; }
  .nt { color: navy; }
  .nv { color: teal; }
  .ow { color: #000000; font-weight: bold; }
  .w { color: #bbb; }
  .mf, .mh, .mi, .mo { color: #099; }
  .sb, .sc, .sd, .s2, .se, .sh, .si, .sx { color: #d14; }
  .sr { color: #009926; }
  .s1 { color: #d14; }
  .ss { color: #990073; }
  .bp { color: #999; }
  .vc, .vg, .vi { color: teal; }
  .il { color: #099; }
}
