//
//  Font Awesome 3.0.2
//  -------------------------------------------------------
//  - The Font Awesome font is licensed under the SIL Open Font License - http://scripts.sil.org/OFL
//  - Font Awesome CSS, LESS, and SASS files are licensed under the MIT License -
//    http://opensource.org/licenses/mit-license.html
//  - The Font Awesome pictograms are licensed under the CC BY 3.0 License - http://creativecommons.org/licenses/by/3.0/
//  - Attribution is no longer required in Font Awesome 3.0, but much appreciated:
//    "Font Awesome by Dave Gandy - http://fortawesome.github.com/Font-Awesome"
//

@font-face {
  font-family: 'FontAwesome';
  src: url('#{$FontAwesomePath}/fontawesome-webfont.eot?v=3.0.1');
  src: url('#{$FontAwesomePath}/fontawesome-webfont.eot?#iefix&v=3.0.1') format('embedded-opentype'),
    url('#{$FontAwesomePath}/fontawesome-webfont.woff?v=3.0.1') format('woff'),
    url('#{$FontAwesomePath}/fontawesome-webfont.ttf?v=3.0.1') format('truetype');
//  url('#{$FontAwesomePath}/fontawesome-webfont.svg#fontawesomeregular?v=3.0.1') format('svg');
//  src: url('#{$FontAwesomePath}/FontAwesome.otf') format('opentype');
  font-weight: normal;
  font-style: normal;
}

//  Font Awesome styles
//    -------------------------------------------------------
[class^="icon-"],
[class*=" icon-"] {
  font-family: FontAwesome;
  font-weight: normal;
  font-style: normal;
  text-decoration: inherit;
  -webkit-font-smoothing: antialiased;

  display: inline;
  width: auto;
  height: auto;
  line-height: normal;
  vertical-align: baseline;
  background-image: none;
  background-position: 0% 0%;
  background-repeat: repeat;
  margin-top: 0;
}

[class^="icon-"]:before,
[class*=" icon-"]:before {
  text-decoration: inherit;
  display: inline-block;
  speak: none;
}

/* makes sure icons active on rollover in links */
a {
  [class^="icon-"],
  [class*=" icon-"] {
    display: inline-block;
  }
}

/* makes the font 33% larger relative to the icon container */
.icon-large:before {
  vertical-align: -10%;
  font-size: 4/3em;
}

// Icon Borders
// -------------------------

.icon-border {
  border: solid 1px $borderColor;
  padding: .2em .25em .15em;
  @include border-radius(3px);
}

[class^="icon-"],
[class*=" icon-"] {
  &.pull-left {
    margin-right: .3em;
  }
  &.pull-right {
    margin-left: .3em;
  }
}

/*  Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen
    readers do not read off random characters that represent icons */
.icon-info-sign:before    { content: "\f023"; }
.icon-chevron-down:before   { content: "\f024"; }
.icon-comments:before   { content: "\f025"; }
.icon-bullhorn:before   { content: "\f027"; }
.icon-signout:before    { content: "\f026"; }
.icon-user:before    { content: "\f021"; }
.icon-cog:before    { content: "\f022"; }
